<?php
/**
 * Staffing Engine - Chatbot - Admin class
 *
 * @link        https://staffing-engine-chatbot.ai/
 * @since       0.2.0
 *
 * @package     Staffing Engine - Chatbot
 * @author      Staffing Engine
 * @copyright   (c) 2022 staffing Engine
 *
 * @license     https://opensource.org/licenses/GPL-3.0 GNU General Public License version 3
 */

namespace StaffingEngine\Chatbot;

use StaffingEngine\Chatbot\Bin\RationalOptionPages;
use \WP_Post;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Configure plugin in wp admin
 *
 * @since 0.2.0
 */
final class Admin {

  /**
	 * Instance of this class.
	 *
	 * @since    0.2.0
	 * @var      object
	 */
	protected static $instance = null;


  /**
	 * Configuration options for the plugin
	 *
	 * @since    0.3.0
	 * @var      array
	 */
  private static $options = array();

  /**
   * Pages registered to the site
   *
   * @since 0.3.0
   * @var array
   */
  private static $site_pages = array();

  /**
	 * Instance Builder
	 *
	 * Singleton pattern means we create only one instance of the class.
	 *
   * @since 0.2.0
   * @return object
   */
  public static function get_instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self;
    }

    return self::$instance;
  }

	/**
	 * Constructor
	 *
	 * @since 0.2.0
	 */
	public function __construct() {
    $this->register_hooks();

    self::$options = get_option( Plugin::SLUG . '_settings', array() );

    // Utils\debug_log('Chatbot Admin loaded with config:');
    // Utils\debug_log(print_r( self::$options, true ) );

    self::$site_pages = $this->get_site_pages();

    $this->build_settings_page(self::$site_pages);
  }

  /**
   * Register hooks
   *
   * Tell wordpress to enqueue scripts, set up admin pages, and modify the UI
   *
   * @since 0.2.0
   */
  private function register_hooks() {
    $should_notify = $this->should_notify_unsupported_php();
    if ( $should_notify ) {
      add_action( 'admin_notices', array( $this, 'notify_unsupported_php' ) );
    }

    add_action( 'add_option_' . Plugin::SLUG . '_settings', array( $this, 'set_default_options' ), 10, 2 );

    add_action( 'admin_enqueue_scripts', array( $this, 'load_setting_page_styles' ) );
    add_action( 'admin_enqueue_scripts', array( $this, 'load_setting_page_scripts' ) );

    add_filter( 'plugin_action_links_' . SE_CHAT_BASENAME , array( $this, 'add_plugin_links' ), 10, 1 );
  }

  /**
   * Load settings page styles
   *
   * @since 0.2.0
   */
  public function load_setting_page_styles($hook) {
    if ( 'settings_page_staffing_engine_chatbot' != $hook ) {
      return;
    }

    wp_enqueue_style( Plugin::SLUG . '-settings-styles' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'css/se-chatbot-admin.css'), array(), null );
  }

  /**
   * Load settings page scripts
   *
   * @since 0.2.0
   */
  public function load_setting_page_scripts($hook) {
    if ( 'settings_page_staffing_engine_chatbot' != $hook ) {
      return;
    }

    wp_enqueue_script( Plugin::SLUG . '-settings-scripts' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'js/se-chatbot-admin.js' ), array(), null, true );
  }

  /**
   * Add custom links on the plugins page
   *
   * @param  array  $links
   * @since  0.2.0
   * @return array
   */
  public function add_plugin_links( $links ) {
    $new_links = array(
      'settings' => sprintf( __( '<a href="%s">Settings</a>', Plugin::SLUG ), esc_url( admin_url( 'options-general.php?page=staffing_engine_chatbot' ) ) ),
    );

    return array_merge( $new_links, $links );
  }

  /**
   * When the plugin options are first saved we can add some default config values that can't be set via RationalOptionPages
   *
   * @since 0.3.0
   */
  public function set_default_options(string $option, array $value) {
    // Default values to pre-load on initial plugin install.
    $defaults = array(
      'invite_message' => 'Hey! I\'m the chatbot. 👋 I can help you find the right job 😊',
    );

    $updatedOptions = array_merge( $value, $defaults );

    update_option( Plugin::SLUG . '_settings', $updatedOptions );
  }

  /**
   * Build the settings page UI
   *
   * @since 0.2.0
   */
  private function build_settings_page($pages) {
    $page_choices = array();
    foreach ($pages as $page) {
      $value = $page['title'];
      $page_choices[$page['id']] = $value;
    }

    $pageConfig = array(
			Plugin::SLUG . '_settings'	=> array(
				'parent_slug' => 'options-general.php',
				'page_title'	=> __( Plugin::NAME, Plugin::SLUG ),
        'menu_title' => __( 'Staffing Engine', Plugin::SLUG ),
				'sections' => array(
					'config' => array(
						'title'	 => __( 'Configuration', Plugin::SLUG ),
						'fields' => array(
              'enable' => array(
                'id'      => 'enable',
                'type'    => 'checkbox',
                'title'   => __( 'Enable', Plugin::SLUG ),
                'checked' => $this->get_checkbox_value('enable'),
              ),
              'rendering' => array(
                'id'		=> 'rendering',
                'title'	=> __( 'Display on', Plugin::SLUG ),
                'text'  => __( 'Specify which pages the chatbot renders on', Plugin::SLUG ),
                'type'  => 'select',
                'value' => 'all',
                'choices'		=> array(
                  'all'      => __( 'All pages', Plugin::SLUG ),
                  'except'   => __( 'All pages except', Plugin::SLUG ),
                  'only'     => __( 'Only on selected pages', Plugin::SLUG ),
                ),
              ),
              // This field will be hidden by default, and js will render it when the appropriate 'rendering' setting is set
              'rendering_except' => array(
                'id'      => 'rendering_except',
                'type'		=> 'select',
								'title'		=> __( 'Exception List', Plugin::SLUG ),
                'text'    => __( 'FYI: Cmd/Ctrl + Click to select individual items, Shift + Click to select multiple items at once', Plugin::SLUG ),
								'value' 	=> array(),
                'choices' => $page_choices,
                'attributes' => array(
                  'multiple' => 'true',
                ),
              ),
              // This field will be hidden by default, and js will render it when the appropriate 'rendering' setting is set
              'rendering_only' => array(
                'id'      => 'rendering_only',
                'type'		=> 'select',
                'title'		=> __( 'Only List', Plugin::SLUG ),
                'text'    => __( 'FYI: Cmd/Ctrl + Click to select individual items, Shift + Click to select multiple items at once', Plugin::SLUG ),
                'value' 	=> array(),
                'choices' => $page_choices,
                'attributes' => array(
                  'multiple' => 'true',
                ),
              ),
							'key' => array(
								'id'		  => 'key',
								'type'		=> 'text',
								'title'		=> __( 'Bot key', Plugin::SLUG ),
                'text'    => __( 'Your unique chat instance key. Contact Staffing Engine support if you haven\'t received one.', Plugin::SLUG ),
								'default'	=> '',
							),
            ),
          ),
          'appearance' => array(
            'title'  => __( 'Appearance', Plugin::SLUG),
            'text'   => __( 'Customize the chat widget\'s appearance', Plugin::SLUG ),
            'fields' => array(
              'position' => array(
                'id'		  => 'position',
                'title'	  => __( 'Bot position', Plugin::SLUG ),
                'text'    => __( 'Set the location of the bot on your site.', Plugin::SLUG ),
                'type'    => 'select',
                'value'   => 'bottom-right',
                'choices'	=> array(
                  'bottom-right' => __( 'Bottom Right', Plugin::SLUG ),
                  'bottom-left'  => __( 'Bottom Left', Plugin::SLUG ),
                  'top-right'    => __( 'Top Right', Plugin::SLUG ),
                  'top-left'     => __( 'Top Left', Plugin::SLUG ),
                ),
              ),
              'widgetColor'	=> array(
                'id'    => 'widget_color',
                'title'	=> __( 'Widget Color', Plugin::SLUG ),
                'text'  => __( 'Set the background color of the chatbot embed.', Plugin::SLUG ),
                'type'	=> 'color',
                'value'	=> '#c4c4c4',
              ),
              'thumbLogo'	=> array(
                'id'    => 'thumb_logo',
                'title'	=> __( 'Thumb Logo', Plugin::SLUG ),
                'text'  => __( 'Upload a custom logo for your chatbot.', Plugin::SLUG ),
                'type'	=> 'media',
                'value'	=> '',
              ),
              'animation' => array(
                'id'      => 'animation',
                'title'	  => __( 'Animation type', Plugin::SLUG ),
                'text'    => __( 'Choose an animation style to get the attention of site visitors.', Plugin::SLUG ),
                'type'	  => 'select',
                'value'	  => 'pulse',
                'choices'	=> array(
                  'pulse' => __( 'Pulse', Plugin::SLUG ),
                  'ring'  => __( 'Ring', Plugin::SLUG ),
                  'zoom'  => __( 'Zoom', Plugin::SLUG ),
                ),
              ),
            ),
          ),
          'invite_popup' => array(
            'title'  => __( ' Invitation Popup', Plugin::SLUG ),
            'text'   => __( 'Show an informational popup next to the widget after a set duration', Plugin::SLUG ),
            'fields' => array(
              'inviteEnable' => array(
                'id'      => 'invite_enable',
                'type'    => 'checkbox',
                'title'   => __( 'Enable Invitation Popup', Plugin::SLUG ),
                'checked' => $this->get_checkbox_value('invite_enable'),
              ),
              'inviteMessage'	=> array(
                'id'    => 'invite_message',
                'title'	=> __( 'Invite Message', Plugin::SLUG ),
                'type'	=> 'textarea',
                'class' => 'inviteMessage',
                'rows'  => 5,
                'text'	=> __( 'Text shown to promote your chatbot in the popup.', Plugin::SLUG ),
                'value'	=> '',
              ),
              'inviteButtonText' => array(
                'id'    => 'invite_button_text',
                'title'	=> __( 'Invite Button text', Plugin::SLUG ),
                'text'  => __( 'Display a button with this text in the popup (optional).', Plugin::SLUG ),
                'type'  => 'text',
                'value' => '',
              ),
              'inviteImage' => array(
                'id'    => 'invite_image',
                'title'	=> __( 'Invite Image', Plugin::SLUG ),
                'text'  => __( 'Display an image in the popup (optional).', Plugin::SLUG ),
                'type'  => 'media',
                'value' => '',
              ),
              'inviteShowCloseIcon' => array(
                'id'      => 'invite_show_close_icon',
                'title'	  => __( 'Show Close Icon', Plugin::SLUG ),
                'text'    => __( 'Show the close icon on the popup. If not shown, the popup cannot be dismissed.', Plugin::SLUG  ),
                'type'    => 'radio',
                'value'   => 'yes',
                'choices'		=> array(
                  'yes'	=> __( 'Show', Plugin::SLUG),
                  'no'	=> __( 'Don\'t Show', Plugin::SLUG ),
                ),
              ),
              'inviteTimeout'	=> array(
                'id'         => 'invite_timeout',
                'title'	     => __( 'Invite Timeout', Plugin::SLUG ),
                'text'       => __( 'Set how long until the popup appears (in seconds)', Plugin::SLUG ),
                'type'	     => 'number',
                'value'	     => 6,
                'attributes' => array(
                  'min' => 1,
                  'max' => 60,
                )
              ),
            ),
          ),
          'autoExpand' => array(
            'title'  => __( 'Auto Expand', Plugin::SLUG),
            'text'   => __( 'Automatically expand the chatbot sometime after the site has loaded', Plugin::SLUG ),
            'fields' => array(
              'autoExpandEnable' => array(
                'id'      => 'auto_expand_enable',
                'title'   => __( 'Enable Auto Expand', Plugin::SLUG ),
                'text'    => __( 'Auto Expand will automatically open the chatbot window after a set amount of time. Use "Enable, excluding small screens" to prevent auto expand from taking over the entire site on mobile devices.', Plugin::SLUG ),
                'type'    => 'select',
                'value'   => 'disable',
                'choices' => array(
                  'disable'           => __( 'Disable', Plugin::SLUG ),
                  'enable-not-mobile' => __( 'Enable, excluding small screens', Plugin::SLUG ),
                  'enable'            => __( 'Enable', Plugin::SLUG ),
                ),
              ),
              'autoExpandDelay'	=> array(
                'id'	       => 'auto_expand_delay',
                'title'	     => __( 'Auto Expand Delay', Plugin::SLUG ),
                'text'       => __( 'How long to wait (in seconds) before the chatbot window automatically opens.', Plugin::SLUG ),
                'type'	     => 'number',
                'value'	     => 5,
                'attributes' => array(
                  'min' => 1,
                  'max' => 60,
                )
              ),
            ),
          ),
          'behavior' => array(
            'title'  => __( 'Behavior', Plugin::SLUG),
            'text'   => __( 'Customize how the chatbot acts under certain conditions', Plugin::SLUG ),
            'fields' => array(
              'allowStartNewConversation'		=> array(
                'id'      => 'allow_start_new_conversation',
                'title'   => __( 'Allow starting a new conversation', Plugin::SLUG ),
                'text'    => __( 'Set whether site visitors can start a new conversation or are required to pick up from a previous conversation, if one exists.', Plugin::SLUG ),
                'type'    => 'radio',
                'value'   => 'yes',
                'choices' => array(
                  'yes'	=> __( 'Allow', Plugin::SLUG),
                  'no'	=> __( 'Don\'t Allow', Plugin::SLUG ),
                ),
              ),
            ),
          ),
				),
			),
		);

    new RationalOptionPages( $pageConfig );
  }

  /**
   * Check if we should notify about an unsupported PHP version
   *
   * @since 0.2.0
   * @return boolean
   */
  private function should_notify_unsupported_php() {
    return version_compare( PHP_VERSION, '7.1.0', '<=' );
  }

  /**
   * Admin Notice if Unsupported PHP Version. PHP Versions below this might have issues running the plugin.
   *
   * @since 0.2.0
   */
  public function notify_unsupported_php() {
    $html = '<div class="notice notice-error is-dismissible"><p>%1$s</p></div>';

    $message = __( '%s requires PHP Version %s or better. Contact your web host to upgrade so you can use the features offered by this plugin.', Plugin::SLUG );

    printf( $html, esc_html( sprintf( $message, Plugin::NAME, '7.1.0' ) ) );
  }

  /**
   * Get the value of a checkbox option, returning false if no value is stored
   *
   * Checkboxes return `'on'` when checked, and `false` when unchecked.
   *
   * @since 0.3.0
   * @return boolean
   */
  private function get_checkbox_value($key) {
    $stored_value = array_key_exists( $key, self::$options ) ? self::$options[$key] : false;

    if ( $stored_value == 'on' ) {
      return true;
    }

    return false;
  }

  /**
   * Get pages from the site
   *
   * @since 0.3.0
   * @return array
   */
  private function get_site_pages() {
    $pages = get_pages( array(
      'parent' => 0,
      'post_status' => array( 'publish', 'draft' ),
    ) );

    $map_page_data = function(WP_Post $page) {
      return array(
        'id' => $page->ID,
        'title' => $page->post_title,
        'status' => $page->post_status,
        'guid' => $page->guid,
      );
    };

    return array_map($map_page_data, $pages );
  }
}
